/*****************************************************************************
 *
 * utouch-frame - Touch Frame Library
 *
 * Copyright (C) 2011 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ****************************************************************************/

#include <assert.h>

#include "v2/event.h"
#include "v2/frame.h"

namespace utouch {
namespace frame {

UFEvent::UFEvent(UFEventType type, const Value* data, uint64_t time)
    : ref_count_(1) {
  const Value* value;

  value = new Value(type);
  InsertProperty(UFEventPropertyType, value);

  switch (type) {
    case UFEventTypeDeviceAdded:
    case UFEventTypeDeviceRemoved:
      InsertProperty(UFEventPropertyDevice, data);
      break;

    case UFEventTypeFrame:
      InsertProperty(UFEventPropertyFrame, data);
      break;
  }

  value = new Value(time);
  InsertProperty(UFEventPropertyTime, value);
}

void UFEvent::Ref() {
  ++ref_count_;
}

void UFEvent::Unref() {
  --ref_count_;
  if (ref_count_ == 0)
    delete this;
}

UFEvent::~UFEvent() {
  ::UFFrame frame;

  if (GetProperty(UFEventPropertyFrame, &frame) == UFStatusSuccess)
    static_cast<utouch::frame::UFFrame*>(frame)->ReleasePreviousFrame();
}

} // namespace frame
} // namespace utouch

extern "C" {

void frame_event_ref(UFEvent event) {
  static_cast<utouch::frame::UFEvent*>(event)->Ref();
}

void frame_event_unref(UFEvent event) {
  static_cast<utouch::frame::UFEvent*>(event)->Unref();
}

UFStatus frame_event_get_property(UFEvent event, UFEventProperty property,
                                  void *data) {
  return static_cast<const utouch::frame::UFEvent*>(event)->GetProperty(property,
                                                                  data);
}

UFEventType frame_event_get_type(UFEvent event) {
  UFEventType type;
  const utouch::frame::UFEvent* ufevent =
      static_cast<const utouch::frame::UFEvent*>(event);
  UFStatus status = ufevent->GetProperty(UFEventPropertyType, &type);
  assert(status == UFStatusSuccess);
  return type;
}

uint64_t frame_event_get_time(UFEvent event) {
  uint64_t time;
  const utouch::frame::UFEvent* ufevent =
      static_cast<const utouch::frame::UFEvent*>(event);
  UFStatus status = ufevent->GetProperty(UFEventPropertyTime, &time);
  assert(status == UFStatusSuccess);
  return time;
}

} // extern "C"
