/*****************************************************************************
 *
 * utouch-frame - Touch Frame Library
 *
 * Copyright (C) 2011 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ****************************************************************************/

#ifndef UTOUCH_FRAME_FRAME_H_
#define UTOUCH_FRAME_FRAME_H_

#include <map>
#include <memory>
#include <vector>

#include "utouch/frame.h"
#include "v2/property.h"
#include "v2/typedefs.h"

struct UFFrame_ {};

namespace utouch {
namespace frame {

class UFFrame : public UFFrame_, public Property<UFFrameProperty> {
 public:
  UFFrame() {}
  UFFrame(const SharedWindow& window, const SharedUFFrame& prev);

  UFTouch* CopyTouch(UFTouchId touchid, UFTouchState new_state) const;
  bool IsTouchOwned(UFTouchId touchid);
  void UpdateTouch(const SharedUFTouch& touch);
  bool IsEnded() const;
  UFStatus GetPreviousTouchValue(const UFTouch* touch, UFAxisType type,
                                 float* value) const;
  UFStatus GetPreviousTouchProperty(const UFTouch* touch,
                                    UFTouchProperty property, void* value) const;
  UFStatus GetTouchByIndex(unsigned int index, ::UFTouch* touch) const;
  UFStatus GetTouchById(UFTouchId touch_id, ::UFTouch* touch) const;
  void ReleasePreviousFrame();

  UFFrame(const UFFrame&) = delete;
  UFFrame& operator=(const UFFrame&) = delete;

 private:
  SharedUFFrame prev_;
  SharedWindow window_;
  std::vector<SharedUFTouch> touches_array_;
  std::map<UFTouchId, unsigned int> touches_map_;
};

} // namespace frame
} // namespace utouch

#endif // UTOUCH_FRAME_FRAME_H_
