/*****************************************************************************
 *
 * utouch-frame - Touch Frame Library
 *
 * Copyright (C) 2011 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ****************************************************************************/

#ifndef UTOUCH_FRAME_PROPERTY_H_
#define UTOUCH_FRAME_PROPERTY_H_

#include <map>

#include "v2/value.h"

namespace utouch {
namespace frame {

template<typename T>
class Property {
 public:
  Property() {}
  explicit Property(const Property& property) {
    for (const auto& pair : property.properties_) {
      T property = pair.first;
      Value* value = new Value(*pair.second);
      properties_[property] = std::move(UniqueValue(value));
    }
  }

  void InsertProperty(T property, const Value* value) {
    properties_.erase(property);
    properties_[property] = std::move(UniqueValue(value));
  }

  UFStatus GetProperty(T property, void* data) const {
    auto it = properties_.find(property);
    if (it == properties_.end())
      return UFStatusErrorUnknownProperty;

    it->second->GetValue(data);

    return UFStatusSuccess;
  }

  Property& operator=(const Property&) = delete;

 private:
  typedef std::unique_ptr<const Value> UniqueValue;

  std::map<T, UniqueValue> properties_;
};

} // namespace frame
} // namespace utouch

#endif // UTOUCH_FRAME_PROPERTY_H_
