/*****************************************************************************
 *
 * utouch-frame - Touch Frame Library
 *
 * Copyright (C) 2011 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ****************************************************************************/

#ifndef UTOUCH_FRAME_TOUCH_H_
#define UTOUCH_FRAME_TOUCH_H_

#include <stdint.h>

#include <map>
#include <memory>

#include "utouch/frame.h"
#include "v2/property.h"
#include "v2/typedefs.h"

struct UFTouch_ {};

namespace utouch {
namespace frame {

class UFTouch : public UFTouch_, public Property<UFTouchProperty> {
 public:
  UFTouch(UFTouchState state, UFTouchId id, float x, float y,
           uint64_t time);
  UFTouch(const UFTouch& touch, UFTouchState new_state);

  UFTouchId id() const { return id_; }
  UFTouchState state() const { return state_; }

  void SetValue(UFAxisType type, float value);
  UFStatus GetValue(UFAxisType type, float* value) const;

  UFTouch(const UFTouch&) = delete;
  UFTouch& operator=(const UFTouch&) = delete;

 private:
  const UFTouchId id_;
  const UFTouchState state_;
  std::map<UFAxisType, float> values_;
};

} // namespace frame
} // namespace utouch

#endif // UTOUCH_FRAME_TOUCH_H_
